from zeep import Client, Settings, helpers

#defining global variables to be used later
API_KEY = "XXXXXXXXX"
WSDL = 'https://api.price2spy.com/soap?wsdl'

#it's necessary to disable the strict mode in order to avoid it breaking due to UnexpectedElementError (happens if the order of elements is unexpected)
settings = Settings(strict=False, xml_huge_tree=True)

#initializing the client  
client = Client(wsdl= WSDL, settings=settings)

#getting the list of dictionaries that represent categories
currentPricingDataResponse = client.service.getCategories(API_KEY)

#defining a function that returns pricing data for a specific category whose ID is passed to it as a paremeter
def returnCurrentPricingData(categoryId):
    return client.service.getCurrentPricingData(categoryId, API_KEY)

#currentPricingData response is a zeep object so it's necessary to serialize it to a Dict
currentPricingDataResponse = helpers.serialize_object(currentPricingDataResponse)

#Iterating through a list of categories
for x in currentPricingDataResponse:
	#Printing the names of categories 
	print("Category: " + str(x.get('name')))
	print("\n")
	#Requesting current pricing data for each category
	pricingData = returnCurrentPricingData(str(x.get('id')))
	#Iterating through the list of pricing data
	for y in pricingData:
		#Again, serializing the zeep object to a dictionary
		y = helpers.serialize_object(y)
		#Printing the product name of each category
		print("Product: " + y.get('productName'))
		print("\n")
		#Iterating through a list of URLs and writing each one as well as the price for that website
		for z in y['urls']['url']:
			print("Website URL: " + z.get('url'))
			print("Last measured price for that website: " + str(z['lastMeasurement']['price'].get('amount')) + ' ' +z['lastMeasurement']['price'].get('currency'))
			print('\n')



"""
Example response from the test account

Please note that this is just a part of the response as it's far to large. If needed, the code can be modified to write a response to a txt file

Category: Synthesizer

Product: Roland FA-08
Website URL: http://www.gear4music.de/de/Keyboards-and-Klaviere/Roland-FA-08-Musik-Workstation/X2E
Last measured price for that website: 1391.00 EUR


Website URL: https://www.thomann.de/de/roland_fa_08.htm
Last measured price for that website: 1444.00 EUR


Website URL: http://www.musicstore.de/de_DE/EUR/Roland-FA-08-Synthesizer-Workstation/art-SYN0004504-000
Last measured price for that website: 1444.00 EUR


Website URL: https://www.woodbrass.com/de-de/keys-arranger-keyboards-roland-fa-08-p168601.html
Last measured price for that website: 1444.00 EUR


Website URL: http://www.musik-produktiv.de/roland-fa-08.html
Last measured price for that website: 1444.00 EUR


Website URL: https://www.justmusic.de/de-de/keys/synthesizer/workstation/9133981/fa-08.html
Last measured price for that website: 1444.00 EUR


Website URL: https://www.muziker.de/tasteninstrumente/synthesizer-und-workstations/p/213157-roland-FA-08
Last measured price for that website: 1444.00 EUR


Website URL: http://www.session.de/ROLAND-FA-08.html?refKey=vE5codGG6
Last measured price for that website: 1444.00 EUR


Website URL: https://www.musicstore.de/de_DE/EUR/Roland-FA-08-Synthesizer-Workstation/art-SYN0004504-000
Last measured price for that website: 1444.00 EUR


Website URL: https://www.musikhaus-korn.de/en/roland-fa-08-music-workstation/pd/59811?qsq=roland%20fa-08
Last measured price for that website: 1444.00 EUR


Website URL: https://www.bax-shop.nl/synthesizer/roland-fa-08-music-workstation
Last measured price for that website: 1445.00 EUR


Website URL: https://www.bax-shop.de/roland-fa-08-music-workstation
Last measured price for that website: 1489.00 EUR


Website URL: https://www.bax-shop.fr/synthetiseur/roland-fa-08-station-de-travail?q=9000-0008-9681
Last measured price for that website: 1489.00 EUR


Website URL: https://www.klangfarbe.com/artikel.php?a=roland-fa-08-s39069
Last measured price for that website: 1490.00 EUR


Website URL: https://www.thomann.de/es/roland_fa_08.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZXMiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImVzIn0%3D
Last measured price for that website: 1490.00 EUR


Website URL: https://www.thomann.de/fr/roland_fa_08.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZnIiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImZyIn0%3D
Last measured price for that website: 1490.00 EUR


Website URL: https://www.thomann.de/nl/roland_fa_08.htm?ref=intl&shp=eyJjb3VudHJ5IjoibmwiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6Im5sIn0%3D
Last measured price for that website: 1490.00 EUR


Website URL: https://www.musicstore.com/nl_NL/EUR/Roland-FA-08-Synthesizer-Workstation/art-SYN0004504-000
Last measured price for that website: 1490.00 EUR


Website URL: https://www.pourlesmusiciens.com/roland-fa-08-workstation_74507.html
Last measured price for that website: 1490.00 EUR


Website URL: https://www.keymusic.com/nl/item/roland-fa-08-music-workstation/?q=Roland%20FA-08%20&
Last measured price for that website: 1599.00 EUR


Product: MatrixBrute
Website URL: https://www.bax-shop.nl/synthesizer-1/arturia-matrixbrute-analoge-modulaire-synthesizer
Last measured price for that website: 1645.00 EUR


Website URL: https://www.bax-shop.de/arturia-matrixbrute-analoger-modularer-synthesizer
Last measured price for that website: 1645.00 EUR


Website URL: http://www.session.de/ARTURIA-MatrixBrute.html?listtype=search&searchparam=arturia
Last measured price for that website: 1799.00 EUR


Website URL: https://www.justmusic.de/de-de/keys/synthesizer/analog-synthesizer/9158180/matrixbrute-.html
Last measured price for that website: 1849.00 EUR


Website URL: https://www.musicstore.de/de_DE/EUR/Arturia-MatrixBrute/art-SYN0005121-000
Last measured price for that website: 1849.00 EUR


Website URL: https://www.bax-shop.fr/arturia-matrixbrute-synthetiseur-analogique-modulaire.html
Last measured price for that website: 1869.00 EUR


Website URL: https://www.musikhaus-korn.de/en/arturia-matrixbrute-analog-synthesizer/pd/129064?qsq=matrixbrute
Last measured price for that website: 1899.00 EUR


Website URL: https://www.musicstore.de/nl_NL/EUR/Arturia-MatrixBrute-Analog-Synthesizer/art-SYN0005121-000
Last measured price for that website: 1928.70 EUR


Website URL: https://www.musicstore.com/nl_NL/EUR/Arturia-MatrixBrute-Analog-Synthesizer/art-SYN0005121-000
Last measured price for that website: 1928.70 EUR


Website URL: http://www.musicworldbrilon.de/arturiamatrixbrute.html
Last measured price for that website: 1948.60 EUR


Website URL: https://www.woodbrass.com/de-de/synthesizers-analoger-synthesizer-arturia-matrixbrute-p216998.html
Last measured price for that website: 1949.00 EUR


Website URL: https://www.muziker.de/arturia-matrixbrute
Last measured price for that website: 1949.00 EUR


Website URL: https://www.thomann.de/de/arturia_matrixbrute.htm?ref=intl&shp=eyJjb3VudHJ5IjoiZGUiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6ImRlIn0%3D
Last measured price for that website: 1949.00 EUR


Website URL: https://www.klangfarbe.com/artikel.php?a=arturia-matrixbrute-s41481
Last measured price for that website: 1998.00 EUR


Website URL: http://www.keymusic.com/fr/produit/arturia-matrixbrute/
Last measured price for that website: 1999.00 EUR


Website URL: http://www.michenaud.com/p34169/arturia-matrixbrute.php
Last measured price for that website: 1999.00 EUR


Website URL: http://www.pourlesmusiciens.com/arturia-matrixbrute.html
Last measured price for that website: 1999.00 EUR


Website URL: http://www.keymusic.com/nl/item/arturia-matrixbrute/
Last measured price for that website: 1999.00 EUR


Website URL: https://www.thomann.de/fr/arturia_matrixbrute.htm?ref=search_rslt_arturia_380973_15
Last measured price for that website: 1999.00 EUR


Website URL: https://www.thomann.de/es/arturia_matrixbrute.htm?ref=search_rslt_arturia_380973_15
Last measured price for that website: 1999.00 EUR


Website URL: https://www.amazon.com/gp/offer-listing/B01IFFC2YI/ref=dp_olp_new_mbc?ie=UTF8&condition=new
Last measured price for that website: 1999.00 USD


Website URL: https://www.thomann.de/nl/arturia_matrixbrute.htm?ref=intl&shp=eyJjb3VudHJ5IjoibmwiLCJjdXJyZW5jeSI6IjIiLCJsYW5ndWFnZSI6Im5sIn0%3D
Last measured price for that website: 1999.00 EUR


Website URL: http://www.gear4music.de/en/Keyboards-and-Pianos/Arturia-MatrixBrute-Analog-Synthesizer/1GH1
Last measured price for that website: 2148.00 EUR
"""